#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "spi/spi.h"
#include "st7735/ST7735R_TFT.h"

#include "mpu6050/mpu6050.h"
#include "mpu6050/mpu6050registers.h"

#define ADC_AREF_VOLTAGE 3.3
#define MPU_ADDRESS 0x68

#define ADC_INIT_CHANNEL 0
#define DAY 1
#define NIGHT 0
#define START_X ST7735R_WIDTH / 2
#define START_Y ST7735R_HEIGHT / 2
#define OFFSET 0.3

#define MOVEMENT 1
#define SHIP 5
#define ATTACKS 10
#define ENEMY 20

#define NORTH 1
#define EAST 2
#define SOUTH 3
#define WEST 4

#define DEBUG 1

static int ADC_light_value;
static double med_ax, med_ay, med_az;
static int shipx, shipy;

char red_bg;
char green_bg;
char blue_bg;

ISR(ADC_vect)
{
    ADC_light_value = ADC;
}

/*
 * Functia initializeaza convertorul Analog-Digital.
 */
void ADC_init(void)
{
    DDRA &= ~(1 << PA0);
    ADMUX = (1 << REFS0) | (ADC_INIT_CHANNEL << MUX0);
    ADCSRA = (1 << ADEN) | (1 << ADIE) | (5 << ADPS0) | (1 << ADSC);
}

void ADC_read()
{
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | (ADC_INIT_CHANNEL << MUX0);
    ADCSRA |= (1 << ADSC);
}

uint16_t ADC_get()
{
    return ADC_light_value;
}

/*
 * Functia primeste o valoare numerica raw citita de convertul Analog-Digital
 * si calculeaza tensiunea (in volti) pe baza tensiunei de referinta.
 */
double ADC_voltage(int raw)
{
    (void)raw;
    return raw * ADC_AREF_VOLTAGE / 1023.0;
}

void day_change() {
    uint16_t light;

    ADC_read();

    light = ADC_get();
    
    if (light < 150) {
        red_bg = 46;
        green_bg = 53;
        blue_bg = 50;
    } else {
        red_bg = 203;
        green_bg = 204;
        blue_bg = 174;
    }
}

void decide_move(double ax, double ay, double az) {
    double dx;
    double dy;

    dx = med_ax - ax;
    if (dx > OFFSET) {
        shipy += MOVEMENT;
    } else if (dx < -OFFSET){
        shipy -= MOVEMENT;
    }

    dy = med_ay - ay;
    if (dy > OFFSET) {
        shipx += MOVEMENT;
    } else if (dy < -OFFSET) {
        shipx -= MOVEMENT;
    }

    if (shipx >= ST7735R_WIDTH) {
        shipx = ST7735R_WIDTH;
    }

    if (shipx <= 0) {
        shipx = 0;
    }

    if (shipy >= ST7735R_HEIGHT) {
        shipy = ST7735R_HEIGHT;
    }

    if (shipy <= 0) {
        shipy = 0;
    }
}

int main(void)
{
    char buff[100];
    int j;
    int i;

    uint16_t light;

    char world[ST7735R_WIDTH][ST7735R_HEIGHT];
    char red_ship;
    char green_ship;
    char blue_ship;
    char attack;

    double ax, ay, az;
    double gx, gy, gz;

    // Init Interupts
    sei();
    // Init ADC for light sensor
    ADC_init();
    // Init SPI for LCD.
    SPI_init();
    mpu6050_init();

    // Init LCD
    ST7735R_Begin();
    ST7735R_FillRect(0, 0, ST7735R_WIDTH + 1, ST7735R_HEIGHT, 0, 0, 0);

    DDRD  &= ~(1 << PD0);
    PORTD |= (1 << PD0);

    // Init World
    for (i = 0; i < ST7735R_WIDTH; i++) {
        for (j = 0; j < ST7735R_HEIGHT; j++) {
            world[i][j] = 0;
        }
    }

    // Callibrating
    med_ax = 0;
    med_ay = 0;
    med_az = 0;

    for (i = 0; i < 10; i++) {
        mpu6050_getConvData(&ax, &ay, &az, &gx, &gy, &gz);
        med_ax += ax;
        med_ay += ay;
        med_az += az;
    }

    med_ax /= 10;
    med_ay /= 10;
    med_az /= 10;
    shipx = START_X;
    shipy = START_Y;
    red_ship = 130;
    green_ship = 200;
    blue_ship = 90;
    attack = 3;

while(1) {
    // Changes between day and night after light sensor
    if ((PIND & (1 << PD0)) == 0) {
        day_change();
    }
    ST7735R_FillRect(0, 0, ST7735R_WIDTH + 1, ST7735R_HEIGHT + 1, red_bg, green_bg, blue_bg);

    mpu6050_getConvData(&ax, &ay, &az, &gx, &gy, &gz);

#ifdef DEBUG
    sprintf(buff, "x: %lf", ax);
    ST7735R_DrawText(5, 10, buff, 255, 255, 255, red_bg, green_bg, blue_bg);
    
    sprintf(buff, "y: %lf", ay);
    ST7735R_DrawText(5, 30, buff, 255, 255, 255, red_bg, green_bg, blue_bg);

    sprintf(buff, "z: %lf", az);
    ST7735R_DrawText(5, 50, buff, 255, 255, 255, red_bg, green_bg, blue_bg);

    sprintf(buff, "dx: %lf", med_ax - ax);
    ST7735R_DrawText(5, 70, buff, 255, 255, 255, red_bg, green_bg, blue_bg);
    
    sprintf(buff, "dy: %lf", med_ay - ay);
    ST7735R_DrawText(5, 90, buff, 255, 255, 255, red_bg, green_bg, blue_bg);

    sprintf(buff, "dz: %lf", med_az - az);
    ST7735R_DrawText(5, 110, buff, 255, 255, 255, red_bg, green_bg, blue_bg);

    light = ADC_get();
    sprintf(buff, "light: %d", light);
    ST7735R_DrawText(10, 130, buff, 255, 255, 255, red_bg, green_bg, blue_bg);
#endif

    decide_move(ax, ay, med_az);

    ST7735R_FilledCircle(shipx + 12, shipy + 3, 3, red_ship, green_ship, blue_ship);
    ST7735R_FillRect(shipx, shipy, shipx + 12, shipy + 6, red_ship, green_ship, blue_ship);

    for (i = ST7735R_WIDTH - 1; i >= 0; i--) {
        for (j = ST7735R_HEIGHT; j >= 0; j--) {
            switch (world[i][j]) {
            case SHIP:
                break;
            case ATTACKS:
                /*if ((j + 3) < ST7735R_WIDTH) {
                    ST7735R_Line(j, i, 2, i, 200, 50, 0);
                    world[i][j + 2] = ATTACKS;
                }*/
                //world[i][j] = 0;
                break;
            case ENEMY:
                break;
            default:
                break;
            }
        }
    }

/*
    if (!attack) {
        ST7735R_Line(shipx + 13, shipy + 6, shipx + 12 + 2, shipy + 6, 200, 50, 0);
        world[shipx + 13][shipy + 6] = 10;
        world[shipx + 14][shipy + 6] = 10;
        attack = 3; 
    }
    attack--;
*/
    ST7735R_FilledCircle(125, 80, 2, 255, 255, 125);

    ST7735R_Line(20, 30, 20, 32, 200, 50, 0);

    _delay_ms(10);
}
    return 0;
}
